function POTENTIAL( problem_file )

load(problem_file);

if( findobj('type', 'figure') )
   init_campos = campos();
end

n_ts = length(time_steps);

y0 = qv_to_y(q, v);

%if( ~exist('ref_traj') || size(ref_traj, 2)~=n_ts || true )
if( ~exist('ref_traj') || size(ref_traj, 2)~=n_ts )
   fprintf('Recomputing the reference trajectory ...\n');

   ref_traj = integrate_ode('int_reference(', problem_name, time_steps, y0);

   fprintf('saving ...\n');

   save(problem_file, 'ref_traj', '-append');

   fprintf('done\n');
end

[grid_X, grid_Y] = meshgrid(x_range, y_range);

eval_str = ['Z_val = ', problem_name, 'pot( grid_X, grid_Y, false );'];
eval(eval_str);

surf( grid_X, grid_Y, Z_val );
colormap Gray;

hold on;

%%--%
%%--% do quiver plot
%%--%
%%--u = zeros(size(grid_X));
%%--v = zeros(size(grid_Y));
%%--for i=1:size(grid_X, 1)
%%--   for j=1:size(grid_Y, 2)
%%--
%%--      t = 0;
%%--      x = [grid_X(i, j); grid_Y(i, j); 0; 0];
%%--
%%--      eval_str = ['vec = ', problem_name, 'func(t, x);'];
%%--      eval(eval_str);
%%--
%%--      u(i, j) = vec(3);
%%--      v(i, j) = vec(4);
%%--   end
%%--end
%%--
%%--quiver(grid_X, grid_Y, u, v);

campos(init_campos);

plot_bounds.MIN_X = min(x_range);
plot_bounds.MIN_Y = min(y_range);
plot_bounds.MAX_X = max(x_range);
plot_bounds.MAX_Y = max(y_range);

ref_hand = plot_trajectory(ref_traj, problem_name, plot_bounds, 'k-', 'LineWidth', 2);

test_methods = { ...
%'int_rungekutta(''exp euler'',',    'Explizites Euler',  'g'; ...
%'int_rungekutta(''heun'',',         'Heun',             'b'; ...
'int_rungekutta(''runge kutta'',',  'Runge-Kutta',      'r'; ...
%'int_impeuler(',                    'Implizites Euler',  'b'; ...
%'int_rungekutta(''runge2'',',       'Runge 2',          'g'; ...
%'int_rungekutta(''runge kutta half'',',  'Runge-Kutta',      'g'; ...
'', '', ''};

num_meth = size(test_methods, 1)-1;

legend_handles = zeros(1, num_meth);

errors = zeros(num_meth, n_ts);

best_meth = 0;
best_err = inf;
best_traj = [];

ref_norm = norm(ref_traj);

for ind_m = 1:num_meth
   comp_traj = integrate_ode( test_methods{ind_m, 1}, ...
                              problem_name, time_steps, y0);

   errors(ind_m, :) = sqrt( sum((comp_traj-ref_traj).^2, 1) )/ref_norm;

   norm_errors = norm(errors);
   if( norm_errors < best_err )
      best_err = norm_errors;
      best_meth = ind_m;
      best_traj = comp_traj;
   end

   legend_handles(ind_m) = plot_trajectory(comp_traj, problem_name, ...
                           plot_bounds, [test_methods{ind_m, 3}, '--'], ...
                           'LineWidth', 2);
end

%legend([ref_hand, legend_handles], 'Referenz', test_methods{1:ind_m, 2});

if( is_matlab() )

   handle_ball = plot(q(1), q(2), 'y.', ...
                     'MarkerSize', 50, ...
                     'LineWidth', 1);
   set(handle_ball, 'XDataSource', 'x_val' );
   set(handle_ball, 'YDataSource', 'y_val' );
   set(handle_ball, 'ZDataSource', 'z_val' );

   [q, v] = y_to_qv(best_traj);

   for t_ind = [2:n_ts, kron(ones(1, 4), 1:n_ts)]
      t = time_steps(t_ind);

      title(['Time : ', num2str(t), ...
           '; Energy : ', num2str( get_energy(problem_name, ...
                                   q(:, t_ind), v(:, t_ind), t)) ]);

      x_val = max(plot_bounds.MIN_X, min(plot_bounds.MAX_X, q(1, t_ind)));
      y_val = max(plot_bounds.MIN_Y, min(plot_bounds.MAX_Y, q(2, t_ind)));

      eval_str = ['z_val = ', problem_name, 'pot( x_val, y_val, true );'];
      eval(eval_str);

      refreshdata(handle_ball, 'caller');
      drawnow;
   end

end

hold off;

%return;
%
%if( show_error )
%   subplot(1, 2, 2);
%   semilogy(time_steps, ones(size(time_steps))*eps);
%   ylabel('Relativer Fehler');
%   xlabel('Zeit');
%   hold on;
%end
%
%
%if( show_error && is_matlab() )
%   subplot(1, 2, 1);
%   hold off;
%   subplot(1, 2, 2);
%   hold off;
%else
%   hold off;
%end

function hand = plot_trajectory(traj, problem_name, plot_bounds, varargin)

[q, v] = y_to_qv(traj);

tp_x = max(plot_bounds.MIN_X, min(plot_bounds.MAX_X, q(1, :)));
tp_y = max(plot_bounds.MIN_Y, min(plot_bounds.MAX_Y, q(2, :)));

eval_str = ['tp_z = ', problem_name, 'pot( tp_x, tp_y, true );'];
eval(eval_str);

hand = plot3( tp_x, tp_y, tp_z, varargin{:} );


function y = qv_to_y(q, v)

y = [q; v];

function [q, v] = y_to_qv(y)

q = y(1:2, :);
v = y(3:4, :);

function E = get_energy(problem_name, q, v, ts)

x = q(1);
y = q(2);

eval_str = ['E_pot = ', problem_name, 'pot( x, y, false );'];
eval(eval_str);

E = v'*v/2 + E_pot;

